#include ".\particle.h"
int stuck=0;
long myParticles;
//////////////////////////////Particle Class//////////////////////////////////

Particle::Particle(float xPos=0,float yPos=0, float xVelocity=0, float yVelocity=0)
{	
	position.x=xPos;
	position.y=yPos;

	velocity.x=xVelocity;
	velocity.y=yVelocity;

	age=0;
}

//////////////////////////General Space Warp//////////////////////////////////
SpaceWarp::SpaceWarp(const char * id)
{	setID(id);
}
char * SpaceWarp::getID()
{	return ID;
}
void SpaceWarp::setID(const char * id)
{	strcpy_s(ID,64,id);
}
bool SpaceWarp::isDeflector()
{	return deflector;
}

////////////////////////////Field Warp////////////////////////////////////////

SW_Field::SW_Field(char* id,float xAcc,float yAcc) : SpaceWarp(id)
{
	acceleration.x=xAcc;
	acceleration.y=yAcc;
	deflector=false;
}

float& SW_Field::xAcceleration()
{	return acceleration.x;
}
float& SW_Field::yAcceleration()
{	return acceleration.y;
}

int SW_Field::applyWarp(Particle& p)
{	
	p.velocity=p.velocity+acceleration;
	return 1;
}

////////////////////////////Local Field Warp////////////////////////////////////////
SW_LocalField::SW_LocalField(char* id,float xPosition,float yPosition,
						float xAcc,float yAcc,int attenuation) : SW_Field(id,xAcc,yAcc)
{	
	maxDistance=-1;	//negative value means variable is ignored.
	minDistance=-1;	//negative value means variable is ignored.
	position.x=xPosition;
	position.y=yPosition;
	decayOrder= attenuation;
	//deflector=false;	//parent class does this.
}

mathVector position;
int decayOrder;
float maxDistance,minDistance;

float& SW_LocalField::X()
{	return position.x;
}

float& SW_LocalField::Y()
{	return position.y;
}

int& SW_LocalField::Decay()
{	return decayOrder;
}

float& SW_LocalField::MaxInfluence()
{	return maxDistance;
}

float& SW_LocalField::MinInfluence()
{	return minDistance;
}

int SW_LocalField::applyWarp(Particle& p)
{
	float decay=1;
	float distance=(p.position-position);//distance formula using vectors
	if (maxDistance>=0 && distance>maxDistance) 
		return 0;
	if (minDistance>=0 && distance<minDistance) 
		return 0;

	if (decayOrder)
	{
		distance=distance/10;	//divide by 10 to normalise value

        if (distance>1)
            for (int cnt=0;cnt<decayOrder;cnt++)
				decay*=distance;
	}

	p.velocity	=	p.velocity + acceleration*(1/decay);
	return 1;
};

////////////////////////////Speed Multiplier/////////////////////////////////

SW_SpeedMultiplier::SW_SpeedMultiplier(char* id, float value) : SpaceWarp(id)
{	
	multiplier=value;
	maxSpeed=-1;
}

float& SW_SpeedMultiplier::Multiplier()
{	return multiplier;
}

float& SW_SpeedMultiplier::SpeedLimit()
{	return maxSpeed;
}

int SW_SpeedMultiplier::applyWarp(Particle& p)
{	
	if (maxSpeed>=0 && p.velocity > maxSpeed)
		p.velocity= p.velocity.UnitVector()*maxSpeed;
	else
		p.velocity= p.velocity*multiplier;

	return 1;
}

////////////////////////////PointAccelerator/////////////////////////////////

SW_PointAttractor::SW_PointAttractor(char* id,float xPosition,float yPosition,
									 float acceleration,int attenuation) : SpaceWarp(id)
{	
	maxDistance=-1;	//negative value means variable is ignored.
	minDistance=-1;	//negative value means variable is ignored.
	position.x=xPosition;
	position.y=yPosition;
	acc=acceleration;
	decayOrder= attenuation;
	deflector=false;
}

float& SW_PointAttractor::Acceleration()
{	return acc;
}
float& SW_PointAttractor::XMultiplier()
{	return multiplier.x;
}
float& SW_PointAttractor::YMultiplier()
{	return multiplier.y;
}
float& SW_PointAttractor::X()
{
	return position.x;
}
float& SW_PointAttractor::Y()
{	return position.y;
}

int& SW_PointAttractor::Decay()
{	return decayOrder;
}

float& SW_PointAttractor::MaxInfluence()
{	return maxDistance;
}

float& SW_PointAttractor::MinInfluence()
{	return minDistance;
}

int SW_PointAttractor::applyWarp(Particle& p)
{	
	float decay=1;
	float distance=(p.position-position);//distance formula using vectors
	if (maxDistance>=0 && distance>maxDistance) return 0;
	if (minDistance>=0 && distance<minDistance) return 0;

	if (decayOrder)
	{
		distance=distance/10;	//divide by 10 to normalise value

        if (distance>1)
            for (int cnt=0;cnt<decayOrder;cnt++)
				decay*=distance;
	}

	mathVector posVect;
	posVect		=	position-p.position;
	posVect		=	posVect.UnitVector();
	posVect		=	(-acc/decay)*posVect;
	p.velocity	=	p.velocity+posVect;
	return 1;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~Line Deflector~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SW_Deflector_Line::SW_Deflector_Line(char* id,float x1,float y1,float x2, float y2,
									 float bounce) : SpaceWarp(id)
{
    point1.x=x1;
	point1.y=y1;
	point2.x=x2;
	point2.y=y2;
	deflector=true;
	this->bounce=bounce;
	getSlope();
}
float& SW_Deflector_Line::X1()
{	return point1.x;
}
float& SW_Deflector_Line::Y1()
{	return point1.y;
}
float& SW_Deflector_Line::X2()
{	return point2.x;
}
float& SW_Deflector_Line::Y2()
{	return point2.y;
}
float& SW_Deflector_Line::BounceCoeff()
{	return bounce;
}

void SW_Deflector_Line::getSlope()
{
	if (point1.x == point2.x) 
		vertical=true;
	else
	{
		slope	=(point1.y-point2.y)/(point1.x-point2.x);
		kvalue	= point1.y-(slope*point1.x);
	}
	if (point1.x>point2.x)
		unitVect=(point2-point1).UnitVector();
	else
		unitVect=(point1-point2).UnitVector();
}

//This is the new collision method, something wrong with it still...
bool SW_Deflector_Line::getCollision(mathVector pt1,mathVector pt2,mathVector * collidePt )
{
	return _getIntersection(pt1,pt2,this->point1,this->point2,collidePt);
}

//This is the old collision function
//
//bool SW_Deflector_Line::getCollision(mathVector pt1,mathVector pt2,mathVector * collidePt )
//{
//	static mathVector retval;
//
//	if (vertical)
//	{
//		if (  ((pt1.x - point1.x >= 0) && (pt2.x - point1.x < 0))
//			||((pt1.x - point1.x <= 0) && (pt2.x - point1.x > 0)) )
//		{
//            retval.x	=point1.x;
//			retval.y	= (pt1.y*(retval.x-pt2.x)+pt2.y*(pt1.x-retval.x))/(pt1.x-pt2.x);
//				//point1.x*(pt2.y-pt1.y)/(pt2.x-pt1.x);
//			*collidePt	=retval;
//			return true;
//		}
//		else
//			return false;
//	}
//	else
//	{
//		float k1=pt1.y-slope*pt1.x-kvalue;
//		float k2=pt2.y-slope*pt2.x-kvalue;
//		
//		if ( (k1>=0 && k2<0) || (k1<=0 && k2>0) )
//		{
//			if (pt1.x==pt2.x) 
//			{
//				retval.x	=pt1.x;
//				retval.y	=pt1.x*slope;
//				*collidePt	=retval;
//				return true;
//			}
//			else
//			{
//				float tmpslope	=(pt1.y-pt2.y)/(pt1.x-pt2.x);
//				float tmpkvalue	= pt1.y-(tmpslope*pt1.x);
//                retval.x		=(kvalue-tmpkvalue)/(tmpslope-slope);
//				retval.y		=slope*retval.x+kvalue;
//				*collidePt		=retval;
//				return true;
//			}
//		}
//		else
//			return false;
//	}
//}


int SW_Deflector_Line::applyWarp(Particle& p)
{
	static mathVector cpoint;//store collision point
	static mathVector tmp1,tmp2,tmp3;

	bool collides=getCollision(p.position,p.position+p.velocity,&cpoint);
	if (collides)
	{
		//p.position=cpoint;
        tmp1=unitVect*(p.velocity * unitVect);	//component along collision wall
		tmp2=p.velocity - tmp1;					//component perpendicular to collision
		p.velocity=tmp1 + tmp2;					//flip the perpendicular component
		p.velocity=p.velocity* bounce;
		return 3;	//collision detected
	}
	else
		return 2;	//no collision
}

/////////////////////INTER-PARTICLE FORCES/////////////////////////////

SW_ParticleForces::SW_ParticleForces(char* id, Emitter * e,
							SW_PointAttractor* w) : SpaceWarp(id)
{
	pSystem=e;
	skipStep=1;
	currentStep=0;
	pWarp=w;
}

int& SW_ParticleForces::SkipRate()
{	return skipStep;
}
void SW_ParticleForces::setEmitter(Emitter* e)
{	pSystem=e;
}
Emitter* SW_ParticleForces::getEmitter()
{	return pSystem;
}
void SW_ParticleForces::setPWarp(SW_PointAttractor* w)
{	pWarp=w;
}
SW_PointAttractor* SW_ParticleForces::getPWarp()
{	return pWarp;
}

int SW_ParticleForces::applyWarp(Particle& p)
{
	Particle * p2=pSystem->getPArray();

	//pWarp->Acceleration()*=skipStep;

	for (long i=currentStep;i<pSystem->NumberOfParticles();i+=skipStep)
	{
		if (&p==&p2[i])//don't interact with self.
			continue;
		pWarp->X()=p2[i].position.x;
		pWarp->Y()=p2[i].position.y;
		pWarp->applyWarp(p);
	}
	currentStep++;
	if (currentStep==skipStep)
		currentStep=0;
	//pWarp->Acceleration()/=skipStep;

	return 1;
}

////////////////////INTER-PARTICLE COLLISIONS//////////////////////////

SW_ParticleCollisions::SW_ParticleCollisions(char* id, Emitter * e,
							float bounceCoeff) : SpaceWarp(id)
{
	bounce=bounceCoeff;
	pSystem=e;
	skipStep=1;
	currentStep=0;
	thresh=0.001;
}
float& SW_ParticleCollisions::BounceCoeff()
{	return bounce;
}
int& SW_ParticleCollisions::SkipRate()
{	return skipStep;
}
void SW_ParticleCollisions::setEmitter(Emitter* e)
{	pSystem=e;
}
Emitter* SW_ParticleCollisions::getEmitter()
{	return pSystem;
}
float& SW_ParticleCollisions::Threshold()
{	return thresh;
}

int SW_ParticleCollisions::applyWarp(Particle& p)
{
	Particle * p2=pSystem->getPArray();
	mathVector temp;
	for (long i=currentStep;i<pSystem->NumberOfParticles();i+=skipStep)
	{
		if (&p==&p2[i])//don't collide with itself.
			continue;

		if (thresh> (float)(p.position-p2[i].position))
		{
			temp			=p.velocity * bounce;//swap velocities.
			p.velocity		=p2[i].velocity * bounce;
			p2[i].velocity	=temp;
		}
	}
	currentStep++;
	if (currentStep==skipStep)
		currentStep=0;

	return 1;
}

////////////////////////GENERIC EMITTER////////////////////////////////

Emitter::Emitter(long numParticles,long numWarps)
{	
	p=new Particle[numParticles];
	maxParticles=numParticles;
	this->numParticles=0;

	sw=new SpaceWarp* [numWarps+1];//one extra space is allocated for sorting
	maxWarps=numWarps;
	this->numWarps=0;

	for (int i=0;i<maxWarps;i++)
		sw[i]=0;//no spacewarps.

	maxCollisions	=3;//maximum collision iterations per particle
	deflectorStart	=0;//no spacewarps.
	subEmitter		=0;//null: no sub emitter.
	hasImage		=false;
	blend			=false;
	initialized		=true;
}

Emitter::~Emitter()
{	
	if (initialized)
	{
		delete [] p;
		delete [] sw;
	}
}

bool Emitter::addWarp(SpaceWarp * newWarp)
{	
	if (numWarps==maxWarps)
		return false;
	
	if (newWarp->isDeflector())	//if newWarp is a deflector
	{
		sw[numWarps++]=newWarp;
		return true;
	}
	else//this code ensures that all deflectors are applied after any non-deflectors
	{	//ie: deflectors are at the end of the array
		int i=0;
		
		while( i<numWarps && !sw[i]->isDeflector())//find the first deflector spaceWarp
			i++;
		sw[numWarps++]=sw[i];	//and move it to the end of the array,
        sw[i]=newWarp;			//and put the new non-deflector in its place.

		int cnt=0;
		//store the starting index of deflectors
		for (;cnt<numWarps && !sw[cnt]->isDeflector() ;cnt++)
			;
		deflectorStart=cnt;

		return true;
	}
}

bool Emitter::delWarp(char * warpID)
{	
	for (int i=0;i<numWarps;i++)
		if(strcmp(warpID,sw[i]->getID())==0)
		{
			if (sw[i]->isDeflector() || i==numWarps-1)//if a deflector is being deleted
			{							//or the last warp in the array is being deleted
				sw[i]=sw[--numWarps];	//move last spacewarp to location i
				sw[numWarps]=0;			//remove spacewarp from the end of list
				return true;
			}
			else
			{
                int j=i+1;
				while(!sw[j]->isDeflector())//go to last non-deflector
					j++;
				sw[i]=sw[j-1];			//move last non-deflector to deleted location i

				//this code deletes the last non-deflector spacewarp from its original location
				
				sw[j-1]=sw[--numWarps];	//move last spacewarp to location j-1
				sw[numWarps]=0;			//remove spacewarp from the end of list
				int cnt=0;
				//store the starting index of deflectors
				for (;cnt<numWarps && !sw[cnt]->isDeflector() ;cnt++)
					;
				deflectorStart=cnt;

				return true;
			}
		}
	return false;
}

void Emitter::setParticleLimit(long numParticles)
{	
	if (initialized)
		delete p;

	p=new Particle[numParticles];
	maxParticles=numParticles;
	this->numParticles=0;
}

void Emitter::setWarpLimit(long numWarps)
{	
	if (initialized)
		delete sw;

	sw=new SpaceWarp* [numWarps];
	maxWarps=numWarps;
	this->numWarps=0;

	for (int i=0;i<maxWarps;i++)
		sw[i]=0;//no spacewarps

}

Particle * Emitter::getPArray()
{	return p;
}
long Emitter::NumberOfParticles()
{	return numParticles;
}
long Emitter::NumberOfWarps()
{	return numWarps;
}
long Emitter::getParticleLimit()
{	return maxParticles;
}
long Emitter::getWarpLimit()
{	return maxWarps;
}

void Emitter::delParticle(long index)
{
	if (numParticles==0 || index >= numParticles || index < 0)
		return;
	p[index].age=0;
	p[index]=p[--numParticles];
}

void Emitter::simulate()
{
	static long i,j,k,iterCnt;
    static bool collided;
	iterCnt=0;

	//stuck=0;
	
	for (i=0;i<numParticles;i++)
	{
		p[i].age++;
		//p[i].position.x+=p[i].velocity.x;
		//p[i].position.y+=p[i].velocity.y;

		if (p[i].age > lifetime)
		{
			delParticle(i);
			continue;
		}

		for (j=0;j<deflectorStart;j++)//apply all non-deflectors
			sw[j]->applyWarp(p[i]);

		collided=true;
		iterCnt	=0;

		//if a collision occurs, the velocity changes.
		//so, all deflectors must be recomputed for any
		//new collisions
		while (collided && iterCnt<maxCollisions)
		{
			collided=false;
			iterCnt++;

			for (j=deflectorStart;j<numWarps;j++)
			{
				k= sw[j]->applyWarp(p[i]);
				if (k==3)
					collided=true;
			}
		}

		//if too many collisions occur, then it means all collisions have
		//not been resolved yet. So, movement does not occur and the collision
		//computations continue in the next iteration.
		if (iterCnt < maxCollisions)
		{
			p[i].position.x+=p[i].velocity.x;
			p[i].position.y+=p[i].velocity.y;
		}
		else
		{
			stuck++;
		}

		if (subEmitter)
		{
			subEmitter->X() =p[i].position.x;
			subEmitter->Y() =p[i].position.y;
			subEmitter->emitParticle();
		}
	}
}

void Emitter::drawParticles(HDC hdc)
{
	byte rTmp,gTmp,bTmp;
	long oldColor;

	for (long i=0;i<numParticles;i++)
	{
		if (!hasImage)
		{
			rTmp=(byte)((float)(r1-r0)*p[i].age/lifetime) +r0;
			gTmp=(byte)((float)(g1-g0)*p[i].age/lifetime) +g0;
			bTmp=(byte)((float)(b1-b0)*p[i].age/lifetime) +b0;
			if (blend){	//blend function source alpha=0.5
				oldColor=GetPixel(hdc,(int)p[i].position.x,(int)p[i].position.y);
				rTmp=(rTmp + GetRValue(oldColor))>>1;
				gTmp=(gTmp + GetGValue(oldColor))>>1;
				bTmp=(bTmp + GetBValue(oldColor))>>1;
			}
			SetPixel(hdc, (int)p[i].position.x , (int)p[i].position.y, RGB(rTmp,gTmp,bTmp));
		}
		else
		{
			float angle;
			float tmpx=p[i].velocity.x;
			float tmpy=p[i].velocity.y;

			if (tmpx)
				angle=atan(abs( (float)tmpy/tmpx ));
			else
				angle=piBy2;

			int Xfr=framesX * p[i].age/lifetime;
			int Yfr;

			if	(tmpx>=0 && tmpy<=0)
				Yfr=(int)((float)framesY/4 * angle/piBy2);
			else if (tmpx<=0 && tmpy<=0)
				Yfr=(int)((float)framesY/4 * (2 - angle/piBy2));
			else if (tmpx<=0 && tmpy>=0)
				Yfr=(int)((float)framesY/4 * (2 + angle/piBy2));
			else if (tmpx>=0 && tmpy>=0)
                Yfr=(int)((float)framesY/4 * (4 - angle/piBy2));

			drawPic(hdc,(int)p[i].position.x,(int)p[i].position.y,Xfr,Yfr,hasMask);
		}
	}
}

char * Emitter::getID()
{	return ID;
}
void Emitter::setID(char * id)
{	strcpy_s(ID,64,id);
}

void Emitter::setInitColor(int r,int g,int b)
{	r0=r;g0=g;b0=b;
}
void Emitter::setFinalColor(int r,int g,int b)
{	r1=r;g1=g;b1=b;
}
long& Emitter::LifeTime()
{	return lifetime;
}
long& Emitter::MaximumCollisions()
{	return maxCollisions;
}
bool& Emitter::graphicEmitter()
{	return hasImage;
}
bool& Emitter::AlphaBlend()
{	return blend;
}

SpotEmitter * Emitter::SubEmitter()
{	return subEmitter;
}
void Emitter::SubEmitter(SpotEmitter * subemit)
{	subEmitter=subemit;
}

/////////////////////SpotEmitter///////////////////////////////////////

SpotEmitter::SpotEmitter(char* id,long numParticles,long numWarps) 
						: Emitter(numParticles,numWarps)//call base constructor
{
	position.x=0;
	position.y=0;
	LifeTime()=0;
	rate=1;
	initialSpeed=0;
	speedVariation=0;

	startAngle=0;
	endAngle=2*pi;
}

float& SpotEmitter::X()
{	return position.x;
}

float& SpotEmitter::Y()
{	return position.y;
}

long& SpotEmitter::EmitRate()
{	return rate;
}

float& SpotEmitter::pSpeed()
{	return initialSpeed;
}
float& SpotEmitter::pSpeedVar()
{	return speedVariation;
}

void SpotEmitter::setEmitAngle(float astart,float aend)
{
	startAngle=astart;
	endAngle=aend;
}

bool SpotEmitter::emitParticle()
{
	float theta;
	float velTmp;
	Particle * tmp;

	for(int ratecount=0;ratecount<rate;ratecount++)
	{
		theta=Rnd(startAngle,endAngle);
		if (numParticles==maxParticles)
			return false;
		tmp=p+(numParticles++);

		if (speedVariation) 
			velTmp=Rnd(-speedVariation,speedVariation)+initialSpeed;
		else
			velTmp=initialSpeed;

		tmp->position.x=position.x;
		tmp->position.y=position.y;
		tmp->age=0;
		tmp->velocity.x=velTmp*cos(theta);
		tmp->velocity.y=velTmp*sin(theta);
	}
	return true;
}

/////////////////////////////////Box Emitter///////////////////////////////////////
BoxEmitter::BoxEmitter(char* id,long numParticles,long numWarps)
						: Emitter(numParticles,numWarps)//call base constructor
{
	LifeTime()=0;
	rate=1;
}
float& BoxEmitter::X1()
{	return bound1.x;
}
float& BoxEmitter::Y1()
{	return bound1.y;
}
float& BoxEmitter::X2()
{	return bound2.x;
}
float& BoxEmitter::Y2()
{	return bound2.y;
}
long& BoxEmitter::EmitRate()
{	return rate;
}

bool BoxEmitter::emitParticle()
{
	float xRand,yRand;
	Particle * tmp;

	for(int ratecount=0;ratecount<rate;ratecount++)
	{
		if (numParticles==maxParticles)
			return false;

		xRand=Rnd(bound1.x,bound2.x);
		yRand=Rnd(bound1.y,bound2.y);

		tmp=p+(numParticles++);

		tmp->position.x=xRand;
		tmp->position.y=yRand;
		tmp->age=0;
		tmp->velocity.x=0;
		tmp->velocity.y=0;
	}
	return true;
}
