/*Classes Defined:
	WindowClass
	TimerClass
	TimerClassMT - MultiThread version
	ImageClass
	ImageText

  Functions Defined:
	FullScreen(width,height) -go fullscreen temporarily
*/
#include "WinClass.h"

//////BEGIN FUNCTION DEFINITIONS//////

//constructor
WindowClass::WindowClass()
{
	windowLoaded=false;
	
	position.left=0;
	position.top=0;
	position.right=200;
	position.bottom=200;

	Color_back	= 0;
	Color_fore	= 16777215;
	width	= 200;
	height	= 200;
}

//clears back buffer.
void WindowClass::cls()
{	FillRect(bufferDC,&rect,backBrush);
}

// returns Device context handle of window
// DO NOT call this function unless absolutely necessary.
// Window DC is meant to be encapsulated and hidden.
// Draw all data to backbuffer and call drawBuffer();
HDC WindowClass::GetWindowDC()
{	return(WindowClass::hdc);							
}

//returns the window handle (hWnd)
HWND WindowClass::GetHandle()
{	return hWnd;
}

//returns instance
HINSTANCE WindowClass::Instance()
{	return hInst;
}

// these functions set icons for window
void WindowClass::Icon(LPCTSTR IconName)
{	wcex.hIcon=LoadIcon(wcex.hInstance,IconName);		
}
void WindowClass::IconSmall(LPCTSTR IconName)
{	wcex.hIconSm =LoadIcon(wcex.hInstance,IconName);	
}

// set cursor / mouse pointer for window.
void WindowClass::Cursor(LPCTSTR CursorName)
{	wcex.hCursor=LoadCursor(wcex.hInstance,CursorName);	
}


//initialise class name with this before calling
//the initInstance(...) function.
void WindowClass::ClassName(char *classname)
{	szWindowClass=classname;							
}


//initialise window caption with this before calling
//the initInstance(...) function.
void WindowClass::WindowCaption(char *windowName)
{	szTitle=windowName;									
}


/////below are functions to get/set window coordinates//////
int WindowClass::wLeft()
{	return position.left;
	//return position.rcNormalPosition.left;
}

int WindowClass::wTop()
{	return position.top;
	//return position.rcNormalPosition.top;
}

int WindowClass::wWidth()
{	return width;
}

int WindowClass::wHeight()
{	return height;
}

//gives client left value in screen coordinates
int WindowClass::clientLeft()
{	return (int)(position.right-rect.right);
}

//gives client top value in screen coordinates
int WindowClass::clientTop()
{	return (int)(position.bottom-rect.bottom);
}

void WindowClass::wLeft(int leftval)
{
	position.left=(long)leftval;
	position.right=(long)width+(long)leftval;
	if (windowLoaded)
	{
		SetWindowPos(hWnd,HWND_TOP,position.left,position.top,
					 width,height,SWP_SHOWWINDOW);
		GetClientRect(hWnd,&rect);
	}
}

void WindowClass::wTop(int topval)
{
	position.top=(long)topval;
	position.bottom=(long)height+ (long)topval;\
	if (windowLoaded)
	{
		SetWindowPos(hWnd,HWND_TOP,position.left,position.top,
					 width,height,SWP_SHOWWINDOW);
		GetClientRect(hWnd,&rect);
	}
}

void WindowClass::wWidth( int widthval)
{
	position.right=(long)widthval+position.left;
	width=widthval;
	if (windowLoaded) 
	{
		SetWindowPos(hWnd,HWND_TOP,position.left,position.top,
					 width,height,SWP_SHOWWINDOW);
		GetClientRect(hWnd,&rect);
		recreateBuffer();
	}
}

void WindowClass::wHeight( int heightval)
{
	position.bottom=(long)heightval+position.top;
	height=heightval;
	if (windowLoaded) 
	{
		SetWindowPos(hWnd,HWND_TOP,position.left,position.top,
					 width,height,SWP_SHOWWINDOW);
		GetClientRect(hWnd,&rect);
		recreateBuffer();
	}
}


//this resizes the buffer in response to a window resize
void WindowClass::recreateBuffer()
{
	//dump the old trash
	SelectObject(bufferDC,tmpBMP);
	DeleteObject(bufferBMP);
	DeleteObject(tmpBMP);
	DeleteDC(bufferDC);
	
	//make way for the new stuff
	bufferDC=CreateCompatibleDC(hdc);
	bufferBMP=CreateCompatibleBitmap(hdc,width,height);
	tmpBMP=(HBITMAP)SelectObject(bufferDC,bufferBMP);
}

//call this on each resize or move of the window
//to get the new coordinates.
void WindowClass::updateWindowPos()
{	
	GetWindowRect(hWnd,&position);
	width = (int) (position.right-position.left);
	height= (int) (position.bottom-position.top);
	
	GetClientRect(hWnd,&rect);	
	recreateBuffer();
}

//////end of  window coordinate functions//////

POINT WindowClass::wCaptureMouse(int oldX,int oldY)
{
	POINT tempPt;
	GetCursorPos(&tempPt);

	if (hWnd!=GetForegroundWindow()) return tempPt;

	if ((tempPt.x>position.right) && (oldX<=position.right) ) 
	{	
		tempPt.x=oldX;
		SetCursorPos(oldX,oldY);
	}
	else if ((tempPt.x<position.left) && (oldX>=position.left) )
	{	
		tempPt.x=oldX;
		SetCursorPos(oldX,oldY);
	}

	if ((tempPt.y>position.bottom) && (oldY<=position.bottom) )
	{
		tempPt.y=oldY;
		SetCursorPos(oldX,oldY);
	}
	else if ((tempPt.y<position.top) && (oldY>=position.top) )
	{
		tempPt.y=oldY;
		SetCursorPos(oldX,oldY);
	}
/*
	if (!inRect(tempPt.x,tempPt.y,position) && 
		inRect(oldX,oldY,position) )
	{
		tempPt.x=oldX;
		tempPt.y=oldY;
		SetCursorPos(oldX,oldY);
	}
*/
	return tempPt;
}


///////functions to get or set window colors///////
void WindowClass::backColor(long colorval)
{	
	Color_back=colorval;
	DeleteObject(backBrush); //this may cause errors. 
							 //try using if(windowLoaded)
	backBrush=CreateSolidBrush(colorval);
}

long WindowClass::backColor()
{	return WindowClass::backColor;						
}

long WindowClass::foreColor()
{	return WindowClass::foreColor;						
}

void WindowClass::foreColor(long colorval)
{	Color_fore=colorval;						
	SetTextColor(bufferDC,colorval);
}
///////end of color functions////////

//draws text onto the backbuffer/////
void WindowClass::printText(LPCSTR lpstring,int xpos,int ypos)
{	
TextOut( bufferDC, xpos, ypos, lpstring, (int)strlen(lpstring) );
}

//use this rather pathetically named function to 
//draw contents of backbuffer onto the main window.
void WindowClass::drawBuffer()
{	BitBlt(hdc,0,0,width,height,bufferDC,0,0,SRCCOPY);
}

//use this to get the device context handle to the
//backbuffer of the window. all drawings must be made on it.
HDC WindowClass::GetBufferDC()
{	return bufferDC;									
}

// call this to destroy window
// in response to WM_QUIT or WM_DESTROY
void WindowClass::destroyMe()
{	
	SelectObject(bufferDC,tmpBMP);
	DeleteObject(bufferBMP);
	DeleteObject(tmpBMP);
	DeleteDC(bufferDC);
	DestroyWindow(hWnd);
}


// call this function to initialise the window class data
// to default values. 
// note that the cursor and icons are given default values here.
// so all calls to:
// Icon(...);
// IconSmall(...);
// Cursor(...);
// functions must be made AFTER this function is called.
// function also takes the window coordinates and color values.
void WindowClass::InitData (HINSTANCE hInstance,WNDPROC myWndProc,long backcolor,long forecolor,int left,int top,int width,int height)
{
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= myWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	Color_back			= backcolor;
	Color_fore			= forecolor;
	wcex.hbrBackground	= CreateSolidBrush((COLORREF)Color_back); //(HBRUSH)(COLOR_WINDOW+1);
	backBrush			= wcex.hbrBackground;
	wcex.lpszMenuName	= szWindowClass;//(LPCSTR)IDC_WTEST2;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= NULL;

	//hopefully, here, local values of width and height are used.
	position.left	=(long)left;
	position.top	=(long)top;
	position.right	=(long)(width +left);
	position.bottom =(long)(height+top );
	WindowClass::width =width;
	WindowClass::height=height;

}

// finally, call this to actually create a window.
BOOL WindowClass::InitInstance(HINSTANCE hInstance, int nCmdShow, FORMSTYLE formStyle)
{
	DWORD styletmp;
	hInst = hInstance;
	
	RegisterClassEx(&wcex);
	
	if (formStyle==F_BORDERED)
		styletmp=WS_OVERLAPPEDWINDOW;
	else if(formStyle==F_BORDERLESS)
		styletmp=WS_SYSMENU | WS_POPUP;


	hWnd = CreateWindow(szWindowClass, szTitle,styletmp ,
						position.left,position.top,width,height,
						NULL, NULL, hInstance, NULL);

	if (!hWnd)
		return FALSE;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	hdc=GetDC(hWnd);
	bufferDC=CreateCompatibleDC(hdc);
	bufferBMP=CreateCompatibleBitmap(hdc,width,height);
	tmpBMP=(HBITMAP)SelectObject(bufferDC,bufferBMP);
	GetClientRect(hWnd,&rect);
	
	windowLoaded=true;
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////Timer class/////////////////////////////////

//initialise static variable at file scope
float TimerClass::speedXplier=1.00; 

bool TimerClass::Elapsed() // returns true if the interval has elapsed 
{							// since last function call;
	DWORD currTime=GetTickCount();
	if (currTime-oldTime>=interval*speedXplier)
	{
		oldTime=currTime;
		return true;
	}
	else
		return false;
}

DWORD TimerClass::getInterval()	//returns interval of timer
{	return interval;
}

void TimerClass::setInterval(DWORD value) // sets interval of timer
{	interval=value;
}

//////////////////////MultiThreaded Timer/////////////////////////

//initialise static member
int TimerClassMT::numThreads=0;

TimerClassMT::TimerClassMT()
{	suppressed=false;
}

void TimerClassMT::TimerLoop()
{
	DWORD exitVar;

	numThreads++;

	while (running)			//Thread Loop
	{
		if (Elapsed())		//timer functionality
		{
			(* targetFn)();	//call the target function
			if (suppressed)
				oldTime=GetTickCount();
		}
	}

	//formal exit from a thread
	GetExitCodeThread(hThread,(LPDWORD)&exitVar);//gets the thread's exit code
												 //and stores it in exitVar
	CloseHandle(hThread);		//destroys the handle to the thread
	numThreads--;				//decrement thread count
	ExitThread(exitVar);		//Terminate the thread (any line after this is not executed)
}

void TimerClassMT::StartTimer(void (* lpStartAddress)())
{
	//make a pointer to the static ThreadStarter() function
	LPTHREAD_START_ROUTINE fptr =
		(LPTHREAD_START_ROUTINE ) &TimerClassMT::ThreadStarter;

	//Set target external function
	targetFn=lpStartAddress;
	//and create the thread
	running=true;
	hThread=CreateThread(NULL,0,fptr,this, 0,ThreadID);
}

bool TimerClassMT::isrunning()
{	return running;
}

void TimerClassMT::EndTimer()
{	running=false;
}

bool& TimerClassMT::Suppressed()
{	return suppressed;
}

//////////////////////////////////////////////////////////////////////
///////////////////// Graphics Functions./////////////////////////////

void FullScreen(int width, int height)
{
	DEVMODE dmSettings;

	memset(&dmSettings, 0, sizeof(dmSettings));

	if( !EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dmSettings) )
	{
		int a =ENUM_CURRENT_SETTINGS;
		a=a;
		MessageBox(NULL, "Cannot enumerate display settings", NULL, MB_OK);
		return;
	}
	
	dmSettings.dmPelsWidth = width;
	dmSettings.dmPelsHeight= height;

	int result = ChangeDisplaySettings(&dmSettings, CDS_FULLSCREEN);

	if( result != DISP_CHANGE_SUCCESSFUL )
	{
		MessageBox(NULL, "Cannot change display settings", NULL, MB_OK);
		PostQuitMessage(0);
	}
}


/////////////////BEGIN GRAPHICS_H FUNCTIONS////////////////////


ImageClass::ImageClass()
{	
	imageBMP	=NULL;
	maskBMP		=NULL;
	hasMask		=false;
	fileLoaded	=false;
	isReference	=false;
}

ImageClass::~ImageClass()
{
	ImageClass::delPic();
}

bool ImageClass::loadPic(LPCTSTR fname,int _width,int _height,
						 int _framesX,int _framesY,WindowClass *window)
{
	if (fileLoaded)
		delPic();
	else
		fileLoaded=true;

	imageBMP=(HBITMAP)LoadImage(window->Instance() ,fname,IMAGE_BITMAP,
							    _width,_height,LR_LOADFROMFILE);

	if (imageBMP==NULL)
	{
		fileLoaded=false;
		return false;
	}

	imageDC=CreateCompatibleDC(window->GetBufferDC());
	old1=(HBITMAP)SelectObject(imageDC,imageBMP);

	width =_width;
	height=_height;
	framesX=_framesX;
	framesY=_framesY;
	frameWidth =width /framesX;
	frameHeight=height/framesY;

	isReference=false;
	return true;
}

bool ImageClass::loadMask(LPCTSTR fname,WindowClass *window)
{
	if (!fileLoaded)
		return false;

	maskBMP=(HBITMAP)LoadImage(window->Instance()  ,fname,IMAGE_BITMAP,
								width,height,LR_LOADFROMFILE);
	if (maskBMP==NULL)
		return false;

	maskDC=CreateCompatibleDC(window->GetBufferDC());
	old2=(HBITMAP)SelectObject(maskDC,maskBMP);
	hasMask=true;

	return true;
}

void ImageClass::drawPic(HDC dest,int x,int y,int _frameX,int _frameY,bool masked)
{
	bool kval;
	if (masked && hasMask)
	{	
		
		kval=BitBlt(dest,x,y,frameWidth,frameHeight,maskDC,
			   _frameX*frameWidth,_frameY*frameHeight,MERGEPAINT);
		kval=BitBlt(dest,x,y,frameWidth,frameHeight,imageDC,
			   _frameX*frameWidth,_frameY*frameHeight,SRCAND);
	}
	else
		BitBlt(dest,x,y,frameWidth,frameHeight,imageDC,
			   _frameX*frameWidth,_frameY*frameHeight,SRCCOPY);
}

void ImageClass::delPic()
{
	if (fileLoaded)
	{
		DeleteObject(imageBMP);
		SelectObject(imageDC,old1);
		DeleteDC(imageDC);
		if (hasMask) 
		{
			DeleteObject(maskBMP);
			SelectObject(maskDC,old2);
			DeleteDC(maskDC);
		}
	}
}

int ImageClass::getWidth()
{	return width;	
}

int ImageClass::getHeight()
{	return height;
}

int ImageClass::getFrameWidth()
{	return frameWidth;
}

int ImageClass::getFrameHeight()
{	return frameHeight;
}

int ImageClass::getNumFramesX()
{	return framesX;
}
int ImageClass::getNumFramesY()
{	return framesY;
}

bool ImageClass::Masked()
{	return hasMask;
}

bool ImageClass::Reference()
{	return isReference;
}

//the reference shares same memory as source.
//it is slightly different from the default = operator
void ImageClass::makeReference(ImageClass srcImage)
{
	if (fileLoaded)
		delPic();

	imageDC		=srcImage.imageDC;
	imageBMP	=srcImage.imageBMP;
	old1		=srcImage.old1;
	maskDC		=srcImage.maskDC;
	maskBMP		=srcImage.maskBMP;
	old2		=srcImage.old2;
	width		=srcImage.width;
	height		=srcImage.height;
	framesX		=srcImage.framesX;
	framesY		=srcImage.framesY;
	frameWidth	=srcImage.frameWidth;
	frameHeight	=srcImage.frameHeight;
	animSpeed	=srcImage.animSpeed;
	hasMask		=srcImage.hasMask;
	fileLoaded	=false;
	isReference	=true;
}

ImageClass::ImageClass(ImageClass& srcImage)//copy constructor
{

	if (srcImage.fileLoaded == false)
	{
		imageBMP	=NULL;
		maskBMP		=NULL;
		hasMask		=false;
		fileLoaded	=false;
		isReference	=false;
	}
		

	width		=srcImage.width;
	height		=srcImage.height;
	framesX		=srcImage.framesX;
	framesY		=srcImage.framesY;
	frameWidth	=srcImage.frameWidth;
	frameHeight	=srcImage.frameHeight;
	animSpeed	=srcImage.animSpeed;
	hasMask		=srcImage.hasMask;

	imageDC	=CreateCompatibleDC(srcImage.imageDC);
	imageBMP=CreateCompatibleBitmap(imageDC,width,height);
	old1	=(HBITMAP)SelectObject(imageDC,imageBMP);
	BitBlt(imageDC,0,0,width,height,srcImage.imageDC,0,0,SRCCOPY);

	if (srcImage.hasMask)
	{
		maskDC	=CreateCompatibleDC(srcImage.imageDC);
        maskBMP	=CreateCompatibleBitmap(maskDC,width,height);
		old2	=(HBITMAP)SelectObject(maskDC,maskBMP);
		BitBlt(maskDC,0,0,width,height,srcImage.maskDC,0,0,SRCCOPY);
	}

	isReference	=false;
	fileLoaded	=true;
}


//makes an independent copy of source
ImageClass ImageClass::operator = (ImageClass srcImage)
{
	if (srcImage.fileLoaded == false)
		return srcImage;			//basically, do nothing

	if (fileLoaded)
		delPic();

	width		=srcImage.width;
	height		=srcImage.height;
	framesX		=srcImage.framesX;
	framesY		=srcImage.framesY;
	frameWidth	=srcImage.frameWidth;
	frameHeight	=srcImage.frameHeight;
	animSpeed	=srcImage.animSpeed;
	hasMask		=srcImage.hasMask;

	imageDC	=CreateCompatibleDC(srcImage.imageDC);
	imageBMP=CreateCompatibleBitmap(imageDC,width,height);
	old1	=(HBITMAP)SelectObject(imageDC,imageBMP);
	BitBlt(imageDC,0,0,width,height,srcImage.imageDC,0,0,SRCCOPY);

	if (srcImage.hasMask)
	{
		maskDC	=CreateCompatibleDC(srcImage.imageDC);
        maskBMP	=CreateCompatibleBitmap(maskDC,width,height);
		old2	=(HBITMAP)SelectObject(maskDC,maskBMP);
		BitBlt(maskDC,0,0,width,height,srcImage.maskDC,0,0,SRCCOPY);
	}

	isReference	=false;
	fileLoaded	=true;
	return *this;

}

/////////////////////////////////////////////////////////////


void ImageText::loadFont(LPCTSTR fname,int width,int height,char* list,WindowClass * window)
{
	strcpy_s(charlist,70,list);
	loadPic(fname,width,height,(int)strlen(list),1,window);

}
void ImageText::drawText(char *text,HDC hdc,int xpos,int ypos)
{
	int len=(int)strlen(text),pos;
	char * poschar;
	int charwidth=getFrameWidth();
	for (int cnt=0;cnt<len;cnt++)
	{
		poschar=strchr(charlist,text[cnt]);
		pos=(int)(poschar -charlist);
		drawPic(hdc,xpos+(charwidth*cnt),ypos,pos,0,true);
	}
}

