#include "particle.h"

WindowClass Window1;
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

int oldMouseX=100,oldMouseY=100;//previous mouse position
int mouseX=100,mouseY=100;		// current mouse position

extern int stuck;
extern long myParticles;
long simTime,drawTime;
long simTimeOld,drawTimeOld;

SpotEmitter			spot1("Water",1200,9);	//Main emitter.

SW_PointAttractor	absorb1("Absorb1",200,100,-1,4);//Field that attracts nearby particles
													//and follows the mouse.

SW_PointAttractor	repelIP("Repel_IP",200,100,-0.03,2);//particles repel neighbours
SW_PointAttractor	attrtIP("Attrt_IP",200,100,0.003,1);//particles attract neighbours

SW_Field			gravity("Gravity",0,0.3);

SW_SpeedMultiplier	resist1("AirResistance",0.98);	//slow down moving particles

SW_ParticleForces pforceR("PFR",&spot1,&repelIP);	//wrapper object for repelIP
SW_ParticleForces pforceA("PFA",&spot1,&attrtIP);	//wrapper object for attrtIP


//note: While using DeflectorLine, if you change the position
//of the spaceWarp using "wall1.X()=800;wall1.Y()=600" and so on, 
//you must call wall1.getSlope() after all position changes are done.
//usage: wallName, x1,y1,x2,y2,bounce-Coeff
SW_Deflector_Line	wall1("Wall1",0,50,25,150,.6);
SW_Deflector_Line	wall2("Wall2",25,150,75,150,.6);
SW_Deflector_Line	wall3("Wall3",75,150,100,50,.6);
SW_Deflector_Line	wall4("Wall4",100,50,0,50,.6);

TimerClass t1;		//Serial timer for drawing on the screen and clearing it.
//This is not really a timer class, it has to be called explicitly inside main loop: 
// t1.setInterval(100);
// if(t1.elapsed()) { do stuff }

TimerClassMT mt1;	//New Thread for simulations/emitting particles.
//This is a timer running on a separate thread. It runs on its own:
//	mt1.setInterval(60);
//	mt1.StartTimer(&Simulate);

char intText[128];	//to display number of particles emitted

void Simulate()
{
	//spot1.emitParticle();

	simTimeOld=GetTickCount();

	spot1.simulate();
	
	simTime=GetTickCount()-simTimeOld;

	sprintf_s(intText,127,"%ld, X:%d, Y:%d, Sim:%ld, Draw:%ld",
				spot1.NumberOfParticles(),mouseX,mouseY,simTime,drawTime);
}

void initParticleSystem()
{
	t1.setInterval(60);//this timer is for drawing the particles.

	spot1.X()=50;
	spot1.Y()=75;
	spot1.pSpeed()		=(float)0;
	spot1.pSpeedVar()	=0.003;
	spot1.EmitRate()	=1;
	spot1.LifeTime()	=99999;
	spot1.setEmitAngle	(0,2*pi);
	spot1.setInitColor	(0,250,250);
	spot1.setFinalColor	(0,150,250);

	attrtIP.MaxInfluence()=10;
	repelIP.MaxInfluence()=8;
	pforceR.SkipRate()=1;
	pforceA.SkipRate()=1;

	spot1.addWarp(&absorb1);
	spot1.addWarp(&gravity);
	spot1.addWarp(&pforceA);
	spot1.addWarp(&pforceR);
	spot1.addWarp(&resist1);
	spot1.addWarp(&wall1);
	spot1.addWarp(&wall2);
	spot1.addWarp(&wall3);
	//spot1.addWarp(&wall4);

	//Keep this code commented, for performance reasons, but anyway,
	//it illustrates how to display pictures instead of dots.
	//spot1.loadPic("c:\\droplet.bmp",3,3,1,1,&Window1);
	//spot1.loadMask("c:\\droplet_mask.bmp",&Window1);
	//spot1.graphicEmitter()=true;

	mt1.setInterval(60);
	mt1.Suppressed()=false;
	mt1.StartTimer(&Simulate);
}

void paintStuff(WindowClass& windo)
{
//	static PAINTSTRUCT paint1;

//	BeginPaint(windo.GetHandle(),&paint1);

	//Step 1: Draw whatever is needed onto the Window Buffer
	windo.cls();
	spot1.drawParticles(windo.GetBufferDC());
	windo.printText(intText,0,0);

	//Step 2: Transfer buffer contents to the window (double buffering):
	windo.drawBuffer(); 

//	EndPaint(windo.GetHandle(),&paint1);

}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;

    initParticleSystem();

	//Window Initialisation
	Window1.ClassName("ChaosFX2_TestApp");
	Window1.WindowCaption("ChaosFX2");
	Window1.InitData(hInstance,(WNDPROC)WndProc,0,16777215,
								0,0,1024,768);
	Window1.Icon("IDI_ICON1");
	if (!Window1.InitInstance(hInstance, nCmdShow, F_BORDERED)) 
		return FALSE;
	//End Window Initialisation

	while(1)
	{
		if(PeekMessage(&msg,NULL, 0,0,PM_REMOVE))
		{
			if(msg.message==WM_QUIT)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		if (t1.Elapsed())//paint once in 60 ms
		{
			//make the emiter and spacewarp follow mouse pointer
			absorb1.X()=(float)(mouseX-Window1.clientLeft());
			absorb1.Y()=(float)(mouseY-Window1.clientTop());
			spot1.X()= absorb1.X();
			spot1.Y()= absorb1.Y()+5;			


			//oldMouseX=(int)repel.X();
			//oldMouseY=(int)repel.Y();

			//repel.X()=(float)(mouseX-Window1.clientLeft());
			//repel.Y()=(float)(mouseY-Window1.clientTop());
			//
			//mathVector tempAcc(repel.X()-oldMouseX,repel.Y()-oldMouseY);
			//tempAcc=tempAcc.UnitVector() * (float)(.5);
			//repel.xAcceleration()=tempAcc.x;
			//repel.yAcceleration()=tempAcc.y;
			
			drawTimeOld=GetTickCount();
			paintStuff(Window1);
			drawTime=GetTickCount()-drawTimeOld;
		}
	}

	//stop multithreaded timer
	mt1.EndTimer();


//	This code may not work well. If used, the app may not quit at all.
//
//	while(TimerClassMT::threadCount())//wait till all threads are closed.
//		;

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT paint1;
	POINT mousepos;

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			break;

		case WM_MOUSEMOVE:
			GetCursorPos(&mousepos);

			mouseX=(int)mousepos.x;
			mouseY=(int)mousepos.y;
			
			break;

		case WM_WINDOWPOSCHANGED:
			//This method needs to be called to reinit the
			//image buffers when the window is resized.
			Window1.updateWindowPos();
			break;

		case WM_KEYDOWN:
			//Define a getKeyboard function to handle keyboard events.
			//getKeyboard(wParam);
			if (wParam==VK_SPACE)
				spot1.AlphaBlend()=!spot1.AlphaBlend();
			else if (wParam==VK_RETURN)
				spot1.emitParticle();
			break;

		case WM_PAINT:
			
			//This is the paint mechanism to be used with the WindowClass:

			BeginPaint(hWnd,&paint1);

			//Step 1: Draw whatever is needed onto the Window Buffer
			Window1.cls();
			spot1.drawParticles(Window1.GetBufferDC());
			Window1.printText(intText,0,0);

			//Step 2: Transfer buffer contents to the window (double buffering):
			Window1.drawBuffer(); 

			EndPaint(hWnd,&paint1);
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }

	return 0;
}
