#include ".\mathStuff.h"

/////////////////////////mathVector Class///////////////////////////////////
mathVector::mathVector(float a,float b)
{	
	x=a;
	y=b;
}

mathVector mathVector::operator + (mathVector a)
{	return mathVector(a.x + x, a.y + y);
}
mathVector mathVector::operator - (mathVector a)
{	return mathVector(a.x - x, a.y - y);
}

float mathVector::operator * (mathVector a)		//dot product
{	return (a.x * x + a.y * y);
}

mathVector::operator float ()
{	return sqrt(x*x + y*y);
}

mathVector mathVector::UnitVector()
{	
	float tmp=(float)(*this);
	if (tmp==0) 
		return mathVector(0,0);
	return mathVector(x/tmp,y/tmp);
}

mathVector operator * (float a,mathVector b)//scaling friend
{	return mathVector(a*b.x, a*b.y);
}

mathVector operator * (mathVector b,float a)//scaling friend
{	return mathVector(a*b.x, a*b.y);
}

/////////////////////////////////////////////////////////////////////////////
//////////////////////Other Definitions//////////////////////////////////////

//////////////////////////Check for intersection of 2 lines/////////////////

bool _getIntersection(mathVector ptA1, mathVector ptA2, mathVector ptB1,
					  mathVector ptB2,mathVector * collidePt )
{
	//here, (ptA1,ptA2) make the line A, and (ptB1,ptB2) make the line B

	float xDiffA= ptA1.x-ptA2.x; //xDiff is the difference in x values of the
	float xDiffB= ptB1.x-ptB2.x; //2 points that make each line (A and B)
							     //These values are also denominator in slope eqn.

	bool verticalA=(xDiffA==0);	 //if xDiff is zero, the line is vertical
	bool verticalB=(xDiffB==0);

	mathVector retval;

	if (verticalA && verticalB)//if both are vertical
		return false;
	else if (verticalA)//if A is vertical
	{
		retval.x=ptA1.x;
		retval.y=ptA1.x * (ptB1.y-ptB2.y)/xDiffB + ptB1.y;
		//meaning, y= ptA1.x * SlopeB + ptB1.y
	}
	else if (verticalB)//if B is vertical
	{
		retval.x=ptB1.x;
		retval.y=ptB1.x * (ptA1.y-ptA2.y)/xDiffA + ptA1.y;
		//meaning, y= ptB1.x * SlopeA + ptA1.y
	}
	else
	{
		float SA=(ptA1.y-ptA2.y)/xDiffA;//slope of line A
		float SB=(ptB1.y-ptB2.y)/xDiffB;//slope of line B

		//solve for the point of intersection of the 2 lines
		retval.x= (ptA1.y - ptB1.y + SB*ptB1.x - SA*ptA1.x )/(SB-SA);
		retval.y= ptA1.y + SA*(retval.x - ptA1.x);
	}

	if (ptA1.y==ptA2.y)		//Handle the case where horizontal lines are found.
		retval.y=ptA1.y;	//This is needed to eliminate floating point errors
	if (ptB1.y==ptB2.y)		//caused by the above eqns. The actual y value may be
		retval.y=ptB1.y;	//fractionally different from that given by the eqns.

	//if the point of intersection is needed, return it
	if (collidePt)
		*collidePt=retval;

	//note that the intersection point is for the 'lines' and not the 'line segments'.
	//so, if the point of intersection is on both line segments, then return true.
	if (inRect(retval,ptA1,ptA2) && inRect(retval,ptB1,ptB2))
		return true;
	else
		return false;
}

//////////////////////////////RANDOM NUMBER FUNCTION///////////////////////

float Rnd(float min,float max)
{

	float randomval=(float)rand()/0x7fff;
	randomval=randomval*(max-min) + min;
	return randomval;
}


//function to check if a point is in a rectangle
bool inRect(int xVal,int yVal,RECT rect1)
{
	if (xVal < rect1.left || xVal>rect1.right || 
	    yVal < rect1.top  || yVal>rect1.bottom)
		return false;
	else
		return true;
}

//check if point is inside the 2 bounds
bool inRect(mathVector pt1,mathVector bound1,mathVector bound2)
{
	int x1,y1,x2,y2;
	pt1.x=(int)pt1.x;
	pt1.y=(int)pt1.y;
	if (bound1.x<bound2.x){
		x1=(int)bound1.x;
		x2=(int)bound2.x;
	}
	else{
		x1=(int)bound2.x;
		x2=(int)bound1.x;
	}
	if (bound1.y<bound2.y){
		y1=(int)bound1.y;
		y2=(int)bound2.y;
	}
	else{
		y1=(int)bound2.y;
		y2=(int)bound1.y;
	}
	if (pt1.x<x1 || pt1.x>x2 || pt1.y<y1 || pt1.y>y2)
		return false;
	else
		return true;
}

